# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Page to allow the user to login into Ubuntu One."""

import gettext

from PyQt4 import QtGui, QtCore
from twisted.internet import defer

from ubuntu_sso.qt import build_general_error_message
from ubuntu_sso.qt.gui import SSOWizardPage
from ubuntu_sso.logger import setup_logging
from ubuntu_sso.utils.ui import (
    EMAIL_LABEL,
    FORGOTTEN_PASSWORD_BUTTON,
    is_correct_email,
    LOGIN_PASSWORD_LABEL,
    SIGN_IN_BUTTON,
)


_ = gettext.gettext
logger = setup_logging('ubuntu_sso.current_user_sign_in_page')


class CurrentUserSignInPage(SSOWizardPage):
    """Wizard Page that lets a current user Sign into Ubuntu One."""

    userLoggedIn = QtCore.pyqtSignal('QString', 'QString')
    passwordForgotten = QtCore.pyqtSignal()
    userNotValidated = QtCore.pyqtSignal('QString', 'QString')

    def __init__(self, ui, ping_url, *args, **kwargs):
        super(CurrentUserSignInPage, self).__init__(ui, *args, **kwargs)
        self.ping_url = ping_url

        self._signals = {
            'LoggedIn':
             self._filter_by_app_name(self.on_logged_in),
            'LoginError':
             self._filter_by_app_name(self.on_login_error),
             'UserNotValidated':
             self._filter_by_app_name(self.on_user_not_validated),
         }
        self.setup_page()

    def on_user_not_validated(self, *args):
        """Show the validate email page."""
        email = unicode(self.ui.email_edit.text())
        password = unicode(self.ui.password_edit.text())
        self.userNotValidated.emit(email, password)

    @defer.inlineCallbacks
    def setup_page(self):
        """Setup the widget components."""
        self.backend = yield self.get_backend()
        self._set_translated_strings()
        # lets add call backs to be execute for the calls we are interested
        self._setup_signals()
        self._connect_ui()

    # Invalid names of Qt-inherited methods
    # pylint: disable=C0103

    def nextId(self):
        """Provide the next id."""
        return self.next

    def initializePage(self):
        """Setup UI details."""
        self.setButtonText(QtGui.QWizard.CancelButton,
            _("Cancel"))
        # Layout without custom button 1,
        # without finish button
        self.wizard().setButtonLayout([
            QtGui.QWizard.BackButton,
            QtGui.QWizard.Stretch])

        # Set sign_in_button as default when the page is shown.
        self.ui.sign_in_button.setDefault(True)
        self.ui.sign_in_button.setEnabled(False)
        self.ui.sign_in_button.setProperty("DisabledState",
            not self.ui.sign_in_button.isEnabled())
        self.ui.sign_in_button.style().unpolish(
            self.ui.sign_in_button)
        self.ui.sign_in_button.style().polish(
            self.ui.sign_in_button)

    def _set_translated_strings(self):
        """Set the translated strings."""
        logger.debug('CurrentUserSignInPage._set_translated_strings')
        self.ui.email_label.setText(EMAIL_LABEL)
        self.ui.password_label.setText(LOGIN_PASSWORD_LABEL)
        self.ui.forgot_password_label.setText(FORGOTTEN_PASSWORD_BUTTON)
        self.ui.sign_in_button.setText(SIGN_IN_BUTTON)

    def _connect_ui(self):
        """Connect the buttons to perform actions."""
        logger.debug('CurrentUserSignInPage._connect_buttons')
        self.ui.forgot_password_label.linkActivated.connect(
                                                    self.on_forgotten_password)
        self.ui.email_edit.textChanged.connect(self._validate)
        self.ui.password_edit.textChanged.connect(self._validate)
        self.ui.sign_in_button.clicked.connect(self.login)

    def _validate(self):
        """Perform input validation."""
        valid = True
        correct_mail = is_correct_email(unicode(self.ui.email_edit.text()))
        password = unicode(self.ui.password_edit.text())
        if not correct_mail or not password:
            valid = False
        self.ui.sign_in_button.setEnabled(valid)
        self.ui.sign_in_button.setProperty("DisabledState",
            not self.ui.sign_in_button.isEnabled())
        self.ui.sign_in_button.style().unpolish(self.ui.sign_in_button)
        self.ui.sign_in_button.style().polish(self.ui.sign_in_button)

    def login(self):
        """Perform the login using the self.backend."""
        logger.debug('CurrentUserSignInPage.login')
        # grab the data from the view and call the backend
        email = unicode(self.ui.email_edit.text())
        password = unicode(self.ui.password_edit.text())
        args = (self.app_name, email, password)
        if self.ping_url:
            f = self.backend.login_and_ping
            args = args + (self.ping_url,)
        else:
            f = self.backend.login
        f(*args)

    def on_login_error(self, app_name, error):
        """There was an error when login in."""
        # let the user know
        logger.error('Got error when login %s, error: %s',
            self.app_name, error)
        self.message_box.critical(self, self.app_name,
            build_general_error_message(error))

    def on_logged_in(self, app_name, result):
        """We managed to log in."""
        logger.info('Logged in for %s', app_name)
        email = unicode(self.ui.email_edit.text())
        self.userLoggedIn.emit(app_name, email)
        logger.debug('Wizard.loginSuccess emitted.')

    def on_forgotten_password(self):
        """Show the user the forgotten password page."""
        logger.info('Forgotten password')
        self.passwordForgotten.emit()
