# -*- coding: utf-8 -*-
#
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Test the proxy UI."""

from twisted.internet import defer
from twisted.trial.unittest import TestCase

from ubuntu_sso.qt import proxy_dialog

# pylint: disable=E1101,


class FakeTextWidget(object):
    """A fake widget that contains text."""

    def __init__(self, object_name, called):
        """Create a new instance."""
        self.object_name = object_name
        self.called = called

    # pylint: disable=C0103
    def setText(self, text):
        """Set the text of the widget."""
        if not self.object_name in self.called:
            self.called[self.object_name] = [('setText', text)]
        else:
            self.called[self.object_name].append(('setText', text))

    def setPixmap(self, pixmap):
        """Set a pixmap."""
        if not self.object_name in self.called:
            self.called[self.object_name] = [('setPixmap', pixmap)]
        else:
            self.called[self.object_name].append(('setPixmap', pixmap))

    def setVisible(self, visible):
        """Set the ui element visible."""
        if not self.object_name in self.called:
            self.called[self.object_name] = [('setVisible', visible)]
        else:
            self.called[self.object_name].append(('setVisible', visible))
    # pylint: enable=C0103


class FakeUIProxyCredsDialog(object):
    """The fake .ui for the creds dialog."""

    ui_labels = ('title_label', 'explanation_label', 'connection_label',
                   'error_label', 'username_label', 'password_label',
                   'domain_label', 'logo_label')
    ui_buttons = ('help_button', 'cancel_button', 'save_button')

    def __init__(self):
        """Create a new instance."""
        self.called = {}
        for name in self.ui_labels + self.ui_buttons:
            setattr(self, name, FakeTextWidget(name, self.called))

    # pylint: disable=C0103
    def setupUi(self, dialog):
        """Set the ui."""
        self.called['FakeUIProxyCredsDialog'] = ('setupUi', dialog)
    # pylint: enable=C0103


class FakeQIcon(object):
    """QIcon class."""

    def __init__(self):
        """Create a new instance."""
        self.called = []

    # pylint: disable=C0103
    def fromTheme(self, icon_id):
        """Return the icon with the given id."""
        self.called.append(('fromTheme', icon_id))
        return self
    # pylint: enable=C0103

    # pylint: disable=C0103
    def pixmap(self, h, w):
        """Get the pixmap with the given size."""
        self.called.append(('pixmap', h, w))
        return self
    # pylint: disable=C0103


class ProxyCredsDialogTest(TestCase):
    """Test the proxy creds dialog."""

    @defer.inlineCallbacks
    def setUp(self):
        """Set tests."""
        yield super(ProxyCredsDialogTest, self).setUp()
        self.patch(proxy_dialog, 'Ui_ProxyCredsDialog',
                   FakeUIProxyCredsDialog)
        self.icon = FakeQIcon()
        self.patch(proxy_dialog, 'QIcon', self.icon)

    def test_set_labels(self):
        """Ensure the correct labels are used."""
        dialog = proxy_dialog.ProxyCredsDialog()
        for label in dialog.ui.ui_labels:
            self.assertIn(label, dialog.ui.called, label)

    def test_set_buttons(self):
        """Ensure that buttons are set."""
        dialog = proxy_dialog.ProxyCredsDialog()
        for label in dialog.ui.ui_buttons:
            self.assertIn(label, dialog.ui.called)

    def test_domain_empty(self):
        """Test that if the domain is not passed we set ''"""
        dialog = proxy_dialog.ProxyCredsDialog()
        self.assertIn('domain_label', dialog.ui.called)
        self.assertIn(('setText', ''), dialog.ui.called['domain_label'])

    def test_domain_not_empty(self):
        """Test that we do set the domain."""
        domain = '192.168.1.100'
        dialog = proxy_dialog.ProxyCredsDialog(domain=domain)
        self.assertIn('domain_label', dialog.ui.called)
        self.assertIn(('setText', domain), dialog.ui.called['domain_label'])

    def test_is_error(self):
        """Test that we do set the error label correctly."""
        dialog = proxy_dialog.ProxyCredsDialog(is_error=False)
        self.assertIn('error_label', dialog.ui.called)
        self.assertIn(('setVisible', False), dialog.ui.called['error_label'])

    def test_is_not_error(self):
        """Test that we do set the error label correctly."""
        dialog = proxy_dialog.ProxyCredsDialog(is_error=True)
        self.assertIn('error_label', dialog.ui.called)
        self.assertIn(('setVisible', True), dialog.ui.called['error_label'])

    def test_icon(self):
        """Test that we correctly set the icon."""
        dialog = proxy_dialog.ProxyCredsDialog()
        self.assertIn(('fromTheme', 'gtk-dialog-authentication'),
                      self.icon.called)
        self.assertIn(('pixmap', 48, 48),
                      self.icon.called)
        self.assertIn(('setPixmap', self.icon),
                      dialog.ui.called['logo_label'])
