# -*- coding: utf-8 -*-
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Test the gsettings parser."""

from twisted.trial.unittest import TestCase

from ubuntu_sso.utils.webclient import gsettings

# Some settings are not used as described in:
# https://bugzilla.gnome.org/show_bug.cgi?id=648237

TEMPLATE_GSETTINGS_OUTPUT = """\
org.gnome.system.proxy autoconfig-url '{autoconfig_url}'
org.gnome.system.proxy ignore-hosts {ignore_hosts:s}
org.gnome.system.proxy mode '{mode}'
org.gnome.system.proxy.ftp host '{ftp_host}'
org.gnome.system.proxy.ftp port {ftp_port}
org.gnome.system.proxy.http authentication-password '{auth_password}'
org.gnome.system.proxy.http authentication-user '{auth_user}'
org.gnome.system.proxy.http host '{http_host}'
org.gnome.system.proxy.http port {http_port}
org.gnome.system.proxy.http use-authentication {http_use_auth}
org.gnome.system.proxy.https host '{https_host}'
org.gnome.system.proxy.https port {https_port}
org.gnome.system.proxy.socks host '{socks_host}'
org.gnome.system.proxy.socks port {socks_port}
"""

BASE_GSETTINGS_VALUES = {
    "autoconfig_url": "",
    "ignore_hosts": ["localhost", "127.0.0.0/8"],
    "mode": "none",
    "ftp_host": "",
    "ftp_port": 0,
    "auth_password": "",
    "auth_user": "",
    "http_host": "",
    "http_port": 0,
    "http_use_auth": "false",
    "https_host": "",
    "https_port": 0,
    "socks_host": "",
    "socks_port": 0,
}


class ProxySettingsTestCase(TestCase):
    """Test the getting of the proxy settings."""

    def test_gsettings_cmdline_correct(self):
        """The command line used to get the proxy settings is the right one."""
        expected = "gsettings list-recursively org.gnome.system.proxy".split()
        called = []

        def append_output(args):
            """Append the output and return some settings."""
            called.append(args)
            return TEMPLATE_GSETTINGS_OUTPUT.format(**BASE_GSETTINGS_VALUES)

        self.patch(gsettings.subprocess, "check_output", append_output)
        gsettings.get_proxy_settings()
        self.assertEqual(called[0], expected)

    def test_gsettings_parser_none(self):
        """Test a parser of gsettings."""
        expected = {}
        fake_output = TEMPLATE_GSETTINGS_OUTPUT.format(**BASE_GSETTINGS_VALUES)
        self.patch(gsettings.subprocess, "check_output",
                   lambda _: fake_output)
        ps = gsettings.get_proxy_settings()
        self.assertEqual(ps, expected)

    def test_gsettings_parser_http_anonymous(self):
        """Test a parser of gsettings."""
        template_values = dict(BASE_GSETTINGS_VALUES)
        expected_host = "expected_host"
        expected_port = 54321
        expected = {
            "host": expected_host,
            "port": expected_port,
        }
        template_values.update({
            "mode": "manual",
            "http_host": expected_host,
            "http_port": expected_port,
        })
        fake_output = TEMPLATE_GSETTINGS_OUTPUT.format(**template_values)
        self.patch(gsettings.subprocess, "check_output",
                   lambda _: fake_output)
        ps = gsettings.get_proxy_settings()
        self.assertEqual(ps, expected)

    def test_gsettings_parser_http_authenticated(self):
        """Test a parser of gsettings."""
        template_values = dict(BASE_GSETTINGS_VALUES)
        expected_host = "expected_host"
        expected_port = 54321
        expected_user = "carlitos"
        expected_password = "very secret password"
        expected = {
            "host": expected_host,
            "port": expected_port,
            "username": expected_user,
            "password": expected_password,
        }
        template_values.update({
            "mode": "manual",
            "http_host": expected_host,
            "http_port": expected_port,
            "auth_user": expected_user,
            "auth_password": expected_password,
            "http_use_auth": "true",
        })
        fake_output = TEMPLATE_GSETTINGS_OUTPUT.format(**template_values)
        self.patch(gsettings.subprocess, "check_output",
                   lambda _: fake_output)
        ps = gsettings.get_proxy_settings()
        self.assertEqual(ps, expected)

    def test_gsettings_parser_authenticated_url(self):
        """Test a parser of gsettings with creds in the url."""
        template_values = dict(BASE_GSETTINGS_VALUES)
        expected_host = "expected_host"
        expected_port = 54321
        expected_user = "carlitos"
        expected_password = "very secret password"
        composed_url = '%s:%s@%s' % (expected_user, expected_password,
                                     expected_host)
        expected = {
            "host": expected_host,
            "port": expected_port,
            "username": expected_user,
            "password": expected_password,
        }
        template_values.update({
            "mode": "manual",
            "http_host": composed_url,
            "http_port": expected_port,
            "http_use_auth": "false",
        })
        fake_output = TEMPLATE_GSETTINGS_OUTPUT.format(**template_values)
        self.patch(gsettings.subprocess, "check_output",
                   lambda _: fake_output)
        ps = gsettings.get_proxy_settings()
        self.assertEqual(ps, expected)

    def test_gsettings_auth_over_url(self):
        """Test that the settings are more important that the url."""
        template_values = dict(BASE_GSETTINGS_VALUES)
        expected_host = "expected_host"
        expected_port = 54321
        expected_user = "carlitos"
        expected_password = "very secret password"
        composed_url = '%s:%s@%s' % ('user', 'random',
                                     expected_host)
        expected = {
            "host": expected_host,
            "port": expected_port,
            "username": expected_user,
            "password": expected_password,
        }
        template_values.update({
            "mode": "manual",
            "http_host": composed_url,
            "http_port": expected_port,
            "auth_user": expected_user,
            "auth_password": expected_password,
            "http_use_auth": "true",
        })
        fake_output = TEMPLATE_GSETTINGS_OUTPUT.format(**template_values)
        self.patch(gsettings.subprocess, "check_output",
                   lambda _: fake_output)
        ps = gsettings.get_proxy_settings()
        self.assertEqual(ps, expected)


class ParseProxyHostTestCase(TestCase):
    """Test the parsing of the domain."""

    def test_onlyhost(self):
        """Parse a host with no username or password."""
        sample = "hostname"
        hostname, username, password = gsettings.parse_proxy_host(sample)
        self.assertEqual(username, None)
        self.assertEqual(password, None)
        self.assertEqual(hostname, "hostname")

    def test_user_and_host(self):
        """Parse host just with the username."""
        sample = "username@hostname"
        hostname, username, password = gsettings.parse_proxy_host(sample)
        self.assertEqual(username, "username")
        self.assertEqual(password, None)
        self.assertEqual(hostname, "hostname")

    def test_user_pass_and_host(self):
        """Test parsing a host with a username and password."""
        sample = "username:password@hostname"
        hostname, username, password = gsettings.parse_proxy_host(sample)
        self.assertEqual(username, "username")
        self.assertEqual(password, "password")
        self.assertEqual(hostname, "hostname")

    def test_username_with_at(self):
        """Test parsing the host with a username with @."""
        sample = "username@company.com:password@hostname"
        hostname, username, password = gsettings.parse_proxy_host(sample)
        self.assertEqual(username, "username@company.com")
        self.assertEqual(password, "password")
        self.assertEqual(hostname, "hostname")

    def test_username_with_at_nopass(self):
        """Test parsing the host without a password."""
        sample = "username@company.com@hostname"
        hostname, username, password = gsettings.parse_proxy_host(sample)
        self.assertEqual(username, "username@company.com")
        self.assertEqual(password, None)
        self.assertEqual(hostname, "hostname")

    def test_user_pass_with_colon_and_host(self):
        """Test parsing the host with a password that contains :."""
        sample = "username:pass:word@hostname"
        hostname, username, password = gsettings.parse_proxy_host(sample)
        self.assertEqual(username, "username")
        self.assertEqual(password, "pass:word")
        self.assertEqual(hostname, "hostname")
